/***************************************************************************
                          $FILENAME$  -  description
                             -------------------
    begin                : $DATE$
    copyright            : (C) $YEAR$ by $AUTHOR$
    email                : $EMAIL$
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "envt.hpp"

namespace lib {
  BaseGDL* erf_fun( EnvT* e);
  BaseGDL* errorf_fun( EnvT* e);
  BaseGDL* erfc_fun( EnvT* e);
  BaseGDL* erfi_fun( EnvT* e);
  BaseGDL* erfcx_fun( EnvT* e);
  BaseGDL* dawson_fun( EnvT* e);
  BaseGDL* faddeeva_fun( EnvT* e);
} // namespace
